@@ -10,7 +10,7 @@ n = 1
 h = log.waitfor('spinners running', status = str(n))
 
 hs = []
-print 'type "q" to quit'
+print('type "q" to quit')
 while True:
     s = raw_input('> ')
     if s == 'q':
