@@ -1,20 +1,28 @@
 #!/usr/bin/env python2
+from __future__ import absolute_import
+from __future__ import division
+
 import argparse
+import six
 import string
 import sys
 
+import pwnlib
+pwnlib.args.free_form = False
+
 from pwn import *
+from pwnlib.commandline import common
 
-from . import common
-
-parser = argparse.ArgumentParser(
-    description = "Cyclic pattern creator/finder"
+parser = common.parser_commands.add_parser(
+    'cyclic',
+    help = "Cyclic pattern creator/finder"
 )
 
 parser.add_argument(
     '-a', '--alphabet',
     metavar = 'alphabet',
-    default = string.ascii_lowercase,
+    default = string.ascii_lowercase.encode(),
+    type = six.ensure_binary,
     help = 'The alphabet to use in the cyclic pattern (defaults to all lower case letters)',
 )
 
@@ -50,8 +58,7 @@ group.add_argument(
     help = 'Number of characters to print'
 )
 
-def main():
-    args = parser.parse_args()
+def main(args):
     alphabet = args.alphabet
     subsize  = args.length
 
@@ -59,9 +66,10 @@ def main():
         pat = args.lookup
 
         try:
-            pat = packing.pack(int(pat, 0), subsize*8)
+            pat = int(pat, 0)
         except ValueError:
             pass
+        pat = flat(pat)
 
         if len(pat) != subsize:
             log.critical('Subpattern must be %d bytes' % subsize)
@@ -77,7 +85,7 @@ def main():
             log.critical('Given pattern does not exist in cyclic pattern')
             sys.exit(1)
         else:
-            print offset
+            print(offset)
     else:
         want   = args.count
         result = cyclic(want, alphabet, subsize)
@@ -85,9 +93,11 @@ def main():
         if got < want:
             log.failure("Alphabet too small (max length = %i)" % got)
 
-        sys.stdout.write(result)
+        out = getattr(sys.stdout, 'buffer', sys.stdout)
+        out.write(result)
 
-        if sys.stdout.isatty():
-            sys.stdout.write('\n')
+        if out.isatty():
+            out.write(b'\n')
 
-if __name__ == '__main__': main()
+if __name__ == '__main__':
+    pwnlib.commandline.common.main(__file__)
