@@ -383,7 +383,7 @@ def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, **kw
     >>> # Connect to the SSH server
 
     >>> # Start a process on the server
-    >>> shell = ssh('runner', 'example.pwnme', password='demopass')
+    >>> shell = ssh('travis', 'example.pwnme', password='demopass')
     >>> io = gdb.debug(['bash'],
     ...                 ssh = shell,
     ...                 gdbscript = '''
@@ -564,9 +564,9 @@ def attach(target, gdbscript = '', exe = None, need_ptrace_scope = True, gdb_arg
     ... ''')
 
     >>> # Interact with the process
-    >>> bash.sendline("whoami")
+    >>> bash.sendline("/bin/echo hello")
     >>> bash.recvline()
-    b'runner\n'
+    b'hello\n'
     >>> bash.close()
 
     >>> # Start a forking server
@@ -591,7 +591,7 @@ def attach(target, gdbscript = '', exe = None, need_ptrace_scope = True, gdb_arg
     >>> io.close()
 
     >>> # Connect to the SSH server
-    >>> shell = ssh('runner', 'example.pwnme', password='demopass')
+    >>> shell = ssh('travis', 'example.pwnme', password='demopass')
 
     >>> # Start a process on the server
     >>> cat = shell.process(['cat'])
@@ -649,7 +649,7 @@ def attach(target, gdbscript = '', exe = None, need_ptrace_scope = True, gdb_arg
         if context.os == 'android':
             pidof = adb.pidof
 
-        pids = pidof(target)
+        pids = list(pidof(target))
         if not pids:
             log.error('No such process: %s' % target)
         pid = pids[0]
@@ -866,7 +866,7 @@ def find_module_addresses(binary, ssh=None, ulimit=False):
     Example:
 
     >>> with context.local(log_level=9999):
-    ...     shell =  ssh(host='example.pwnme', user='runner', password='demopass')
+    ...     shell =  ssh(host='example.pwnme', user='travis', password='demopass')
     ...     bash_libs = gdb.find_module_addresses('/bin/bash', shell)
     >>> os.path.basename(bash_libs[0].path)
     'libc.so.6'
