@@ -1,11 +1,20 @@
-from . import keyconsts as kc
-from . import keymap as km
-from . import term
-from . import text
+#!/usr/bin/env python2
+# -*- coding: utf-8 -*-
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
+import six
+import sys
+
+from pwnlib.term import keyconsts as kc
+from pwnlib.term import keymap as km
+from pwnlib.term import term
+from pwnlib.term import text
 
 cursor = text.reverse
 
-buffer_left, buffer_right = u'', u''
+buffer_left, buffer_right = '', ''
 saved_buffer = None
 history = []
 history_idx = None
@@ -27,6 +36,14 @@ suggest_hook = None
 
 tabs = 0
 
+def force_to_bytes(data):
+    if isinstance(data, bytes):
+        return data
+    try:
+        return data.encode('utf-8')
+    except Exception:
+        return data.encode('latin-1')
+
 def set_completer(completer):
     global complete_hook, suggest_hook
     if completer is None:
@@ -78,7 +95,7 @@ def handle_keypress(trace):
 
 def clear():
     global buffer_left, buffer_right, history_idx, search_idx
-    buffer_left, buffer_right = u'', u''
+    buffer_left, buffer_right = '', ''
     history_idx = None
     search_idx = None
     redisplay()
@@ -126,8 +143,8 @@ def self_insert(trace):
 
 def set_buffer(left, right):
     global buffer_left, buffer_right
-    buffer_left = unicode(left)
-    buffer_right = unicode(right)
+    buffer_left = left
+    buffer_right = right
     redisplay()
 
 def cancel_search(*_):
@@ -139,7 +156,7 @@ def cancel_search(*_):
 def commit_search():
     global search_idx
     if search_idx is not None and search_results:
-        set_buffer(history[search_results[search_idx][0]], u'')
+        set_buffer(history[search_results[search_idx][0]], '')
         search_idx = None
         redisplay()
 
@@ -167,7 +184,7 @@ def update_search_results():
 def search_history(*_):
     global buffer_left, buffer_right, history_idx, search_idx
     if search_idx is None:
-        buffer_left, buffer_right = buffer_left + buffer_right, u''
+        buffer_left, buffer_right = buffer_left + buffer_right, ''
         history_idx = None
         search_idx = 0
         update_search_results()
@@ -185,7 +202,7 @@ def history_prev(*_):
         history_idx = -1
     if history_idx < len(history) - 1:
         history_idx += 1
-        set_buffer(history[history_idx], u'')
+        set_buffer(history[history_idx], '')
 
 def history_next(*_):
     global history_idx, saved_buffer
@@ -198,7 +215,7 @@ def history_next(*_):
         saved_buffer = None
     else:
         history_idx -= 1
-        set_buffer(history[history_idx], u'')
+        set_buffer(history[history_idx], '')
 
 def backward_char(*_):
     global buffer_left, buffer_right
@@ -317,11 +334,11 @@ def forward_word(*_):
 
 def go_beginning(*_):
     commit_search()
-    set_buffer(u'', buffer_left + buffer_right)
+    set_buffer('', buffer_left + buffer_right)
 
 def go_end(*_):
     commit_search()
-    set_buffer(buffer_left + buffer_right, u'')
+    set_buffer(buffer_left + buffer_right, '')
 
 keymap = km.Keymap({
     '<nomatch>'   : self_insert,
@@ -351,13 +368,18 @@ keymap = km.Keymap({
     '<any>'       : handle_keypress,
     })
 
-def readline(_size = None, prompt = '', float = True, priority = 10):
+def readline(_size=None, prompt='', float=True, priority=10):
     # The argument  _size is unused, but is there for compatibility
     # with the existing readline
 
     global buffer_handle, prompt_handle, suggest_handle, eof, \
         show_suggestions
 
+    # XXX circular imports
+    from pwnlib.term import term_mode
+    if not term_mode:
+        six.print_(prompt, end='', flush=True)
+        return sys.stdin.readline(_size).rstrip('\n')
     show_suggestions = False
     eof = False
     if prompt:
@@ -376,14 +398,14 @@ def readline(_size = None, prompt = '', float = True, priority = 10):
                     keymap.handle_input()
                 except EOFError:
                     if len(buffer_left + buffer_right) == 0:
-                        return ''
+                        return b''
                 if eof:
-                    return ''
+                    return b''
                 else:
-                    buffer = (buffer_left + buffer_right).encode('utf-8')
+                    buffer = (buffer_left + buffer_right)
                     if buffer:
                         history.insert(0, buffer)
-                    return buffer + '\n'
+                    return force_to_bytes(buffer) + b'\n'
             except KeyboardInterrupt:
                 control_c()
     finally:
@@ -400,10 +422,64 @@ def readline(_size = None, prompt = '', float = True, priority = 10):
         if shutdown_hook:
             shutdown_hook()
 
+def raw_input(prompt='', float=True):
+    r"""raw_input(prompt='', float=True)
+
+    Replacement for the built-in ``raw_input`` using ``pwnlib`` readline
+    implementation.
+
+    Arguments:
+        prompt(str): The prompt to show to the user.
+        float(bool): If set to `True`, prompt and input will float to the
+                     bottom of the screen when `term.term_mode` is enabled.
+    """
+    return readline(None, prompt, float)
+
+def str_input(prompt='', float=True):
+    r"""str_input(prompt='', float=True)
+
+    Replacement for the built-in ``input`` in python3 using ``pwnlib`` readline
+    implementation.
+
+    Arguments:
+        prompt(str): The prompt to show to the user.
+        float(bool): If set to `True`, prompt and input will float to the
+                     bottom of the screen when `term.term_mode` is enabled.
+    """
+    return readline(None, prompt, float).decode()
+
+def eval_input(prompt='', float=True):
+    """eval_input(prompt='', float=True)
+
+    Replacement for the built-in python 2 - style ``input`` using
+    ``pwnlib`` readline implementation, and `pwnlib.util.safeeval.expr`
+    instead of ``eval`` (!).
+
+    Arguments:
+        prompt(str): The prompt to show to the user.
+        float(bool): If set to ``True``, prompt and input will float to the
+                     bottom of the screen when `term.term_mode` is enabled.
+
+    Example:
+
+        >>> try:
+        ...     saved = sys.stdin, pwnlib.term.term_mode
+        ...     pwnlib.term.term_mode = False
+        ...     sys.stdin = io.StringIO(u"{'a': 20}")
+        ...     eval_input("Favorite object? ")['a']
+        ... finally:
+        ...     sys.stdin, pwnlib.term.term_mode = saved
+        Favorite object? 20
+    """
+    from pwnlib.util import safeeval
+    return safeeval.const(readline(None, prompt, float))
+
 def init():
+    global safeeval
     # defer imports until initialization
-    import sys, __builtin__
-    from ..util import safeeval
+    import sys
+    from six.moves import builtins
+    from pwnlib.util import safeeval
 
     class Wrapper:
         def __init__(self, fd):
@@ -414,30 +490,8 @@ def init():
             return self._fd.__getattribute__(k)
     sys.stdin = Wrapper(sys.stdin)
 
-    def raw_input(prompt = '', float = True):
-        """raw_input(prompt = '', float = True)
-
-        Replacement for the built-in `raw_input` using ``pwnlib``s readline
-        implementation.
-
-        Arguments:
-            prompt(str): The prompt to show to the user.
-            float(bool): If set to `True`, prompt and input will float to the
-                         bottom of the screen when `term.term_mode` is enabled.
-        """
-        return readline(None, prompt, float)
-    __builtin__.raw_input = raw_input
-
-    def input(prompt = '', float = True):
-        """input(prompt = '', float = True)
-
-        Replacement for the built-in `input` using ``pwnlib``s readline
-        implementation, and `pwnlib.util.safeeval.expr` instead of `eval` (!).
-
-        Arguments:
-            prompt(str): The prompt to show to the user.
-            float(bool): If set to `True`, prompt and input will float to the
-                         bottom of the screen when `term.term_mode` is enabled.
-        """
-        return safeeval.const(readline(None, prompt, float))
-    __builtin__.input = input
+    if six.PY2:
+        builtins.raw_input = raw_input
+        builtins.input = eval_input
+    else:
+        builtins.input = str_input
